/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */

#include "MTable.hxx"
#include "MColumns.hxx"
#include <connectivity/TKeys.hxx>
#include <connectivity/TIndexes.hxx>


using namespace connectivity::mork;
using namespace connectivity;


OTable::OTable( sdbcx::OCollection* _pTables, OConnection* _pConnection,
                const OUString& Name, const OUString& Type, const OUString& Description )
    :OTable_Base(_pTables, _pConnection, true, Name, Type, Description )
    ,m_pConnection( _pConnection )
{
    construct();
}


sdbcx::OCollection* OTable::createColumns( const ::std::vector< OUString>& _rNames )
{
    return new OColumns( this, m_aMutex, _rNames );
}


sdbcx::OCollection* OTable::createKeys(const ::std::vector< OUString>& _rNames)
{
    return new OKeysHelper( this, m_aMutex, _rNames );
}


sdbcx::OCollection* OTable::createIndexes(const ::std::vector< OUString>& _rNames)
{
    return new OIndexesHelper( this, m_aMutex, _rNames );
}


/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
