/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * This file is part of the LibreOffice project.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/.
 *
 * This file incorporates work covered by the following license notice:
 *
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements. See the NOTICE file distributed
 *   with this work for additional information regarding copyright
 *   ownership. The ASF licenses this file to you under the Apache
 *   License, Version 2.0 (the "License"); you may not use this file
 *   except in compliance with the License. You may obtain a copy of
 *   the License at http://www.apache.org/licenses/LICENSE-2.0 .
 */

#include <tools/gen.hxx>
#include <basegfx/numeric/ftools.hxx>

void Point::RotateAround( Point& rPoint,
                          short nOrientation ) const
{
    long nX = rPoint.X();
    long nY = rPoint.Y();
    RotateAround(nX, nY, nOrientation);
    rPoint.setX(nX);
    rPoint.setY(nY);
}

void Point::RotateAround( long& rX, long& rY,
                          short nOrientation ) const
{
    const long nOriginX = X();
    const long nOriginY = Y();

    if ( (nOrientation >= 0) && !(nOrientation % 900) )
    {
        if ( nOrientation >= 3600 )
            nOrientation %= 3600;

        if ( nOrientation )
        {
            rX -= nOriginX;
            rY -= nOriginY;

            if ( nOrientation == 900 )
            {
                long nTemp = rX;
                rX = rY;
                rY = -nTemp;
            }
            else if ( nOrientation == 1800 )
            {
                rX = -rX;
                rY = -rY;
            }
            else /* ( nOrientation == 2700 ) */
            {
                long nTemp = rX;
                rX = -rY;
                rY = nTemp;
            }

            rX += nOriginX;
            rY += nOriginY;
        }
    }
    else
    {
        double nRealOrientation = nOrientation*F_PI1800;
        double nCos = cos( nRealOrientation );
        double nSin = sin( nRealOrientation );

        // Translation...
        long nX = rX-nOriginX;
        long nY = rY-nOriginY;

        // Rotation...
        rX = + static_cast<long>(nCos*nX + nSin*nY) + nOriginX;
        rY = - static_cast<long>(nSin*nX - nCos*nY) + nOriginY;
    }
}

/* vim:set shiftwidth=4 softtabstop=4 expandtab: */
